api_service_name = "RxuniverseApiSvc"

desc "Register the Rx-Universe API Module as a windows service"
task :register_service do
  begin
    require 'win32/service'
    include Win32
    include RbConfig
    internal_name, display_name, description = api_service_name, 'RxUniverse API', "An application programming interface for RxUniverse"
    root_dir = File.expand_path(File.join(__dir__, '..'))
    service_file = File.expand_path(File.join(root_dir, 'src', 'service.rb'))
    drive = root_dir[0].upcase

    set_dir = "#{drive}: && cd \"#{File.dirname(service_file)}\""
    set_env = "set \"RACK_ENV=production\""
    ruby = File.join(CONFIG['bindir'], CONFIG['ruby_install_name']).tr('/', '\\')
    bundle = File.join(CONFIG['bindir'], "bundle").tr('/', '\\')
    launch_command = "#{set_env} && #{set_dir} && \"#{bundle}\" exec \"#{ruby}\" \"#{service_file}\" \"#{root_dir}\""

    service_bin = "cmd /V /S /C \"#{launch_command}\""

    Service.new(
        service_name: internal_name,
        display_name: display_name || internal_name,
        description: description,
        binary_path_name: service_bin,
        start_type: Service::AUTO_START
    )

    Service.configure(service_name: internal_name, delayed_start: 1)
  rescue Errno::EIO => e
    warn "You may not be running as administrator"
    raise e
  end
end

desc "Unregister a service installed with register_service"
task :unregister_service do
  begin
    require 'win32/service'
    include Win32
    include RbConfig
    internal_name = api_service_name
    if Service.exists? internal_name
      Service.delete internal_name
    else
      warn "#{internal_name} is not currently installed"
    end
  rescue Errno::EIO => e
    warn "You may not be running as administrator"
    raise e
  end
end

desc "Reinstall the Rx-Universe API Service"
task :reregister_service => %w(unregister_service register_service)

desc "Start the Rx-Universe API Service"
task :start_service do
  require 'win32/service'
  include Win32
  Service.start(api_service_name)
end

desc "Stop the Rx-Universe API Service"
task :stop_service do
  require 'win32/service'
  include Win32
  Service.stop(api_service_name)
end