require 'win32/service'
require 'rbconfig'
require 'optparse'
require 'dotenv/load'
include Win32
include RbConfig

options = {}
service_name = "RxuniverseApiSvc"

options_parser = OptionParser.new do |opt|
  opt.on('-i', '--install', 'Install RxUniverse API service') do |i|
    options[:install] = i
  end
  opt.on('-u', '--uninstall', 'Uninstall RxUniverse API service') do |u|
    options[:uninstall] = u
  end
  opt.on('-r', '--reinstall', 'Reinstall RxUniverse API (equivalent to -u -i)') do
    options[:install] = true
    options[:uninstall] = true
  end
end

options_parser.parse!

if options.empty?
  puts options_parser
  exit
end

begin
  if options[:uninstall]
    if Service.exists? service_name
      Service.delete service_name
      puts "#{service_name} uninstalled. If services.msc or similar is open, you may need to close it to complete the uninstallation process"
    else
      puts "#{service_name} is not currently installed"
    end
  end

  if options[:install]
    ruby = File.join(CONFIG['bindir'], CONFIG['ruby_install_name']).tr('/', '\\')
    path = ' "' + File.dirname(File.expand_path($0)).tr('/', '\\')
    path += '\service.rb"'
    cmd = ruby + path
    Service.new(
        service_name: service_name,
        display_name: 'RxUniverse API',
        description: "An application programming interface for RxUniverse",
        binary_path_name: cmd,
        start_type: Service::AUTO_START
    )

    Service.configure(service_name: service_name, delayed_start: 1)
    puts "#{service_name} installed"
  end
rescue Errno::EIO => e
  puts e.message
  puts "Please ensure you're running as Administrator."
rescue SystemCallError => e
  puts e.message
end
