begin
  require 'rubygems'
  require 'win32/daemon'
  require 'rack'
  require 'puma'
  require 'puma/configuration'
  require 'dotenv'
  require 'json'
  include Win32

  Dir.chdir(__dir__ + '\..') do
    Dotenv.load
    $stderr.reopen("./log/service-err.log")
    $stdout.reopen("./log/service.log")
  end


  class RxUniverseDaemon < Daemon

    def service_init
      p 'Service is initializing...'
      dir = File.dirname(File.expand_path($0)).tr('/', '\\')
      Dir.chdir(dir + '\..')
      @puma = Puma::Launcher.new(Puma::Configuration.new do |c|
        c.threads 16, 32
        c.environment "production"

        port = ENV['PORT'] || 4567
        ssl_key_path = ENV['SSL_KEY']
        ssl_certificate_path = ENV['SSL_CERTIFICATE']
        if ssl_key_path && ssl_certificate_path
          c.bind "ssl://0.0.0.0:#{port}?key=#{ssl_key_path}&cert=#{ssl_certificate_path}"
        else
          c.bind "tcp://0.0.0.0:#{port}"
        end
      end, events: Puma::Events.stdio)
    end

    def service_stop
      @puma.halt
      p "Service has received a signal to stop"
    end

    def service_main
      @puma.run
    end
  end

  RxUniverseDaemon.mainloop
rescue StandardError => e
  p "Service failed: #{e}"
end
